#!/usr/bin/python
# kickstart.py - SNAKE kickstart templates
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import pykickstart.parser 
import shlex

header = '''\
# kickstart automatically generated by SNAKE. 
# Please don't modify the following two lines.
# Description: %s
# Version: %s

'''

class Kickstart(object):
    '''A SNAKE kickstart object. Takes an optional argument that selects the
    version of the kickstart language to use. This argument may be a string
    (e.g. 'F7', 'RHEL4') or one of the constants from pykickstart.parser.

    If unspecified, the 'devel' handler class will be used. This is the
    default handler that pykickstart.parser.returnClassForVersion will return
    if called with no argument.

    This class has the following instance variables:
    description: A description of the kickstart object
    version:     The (numeric) version of the kickstart language being used
    handler:     A handler for the kickstart version specified
    parser:      A kickstart parser (uses the aforementioned handler)
    '''
    def __init__(self,version='devel'):
        self.description='Unmodified Kickstart object'
        self.setversion(version)

    def setversion(self,version):
        '''Set this kickstart object to use the kickstart version specified.
        Note that this will reset the handler and parser attributes.'''
        if type(version) == type(str()):
            self.version = pykickstart.parser.stringToVersion(version)
        else:
            self.version = version
        handlerclass = pykickstart.parser.returnClassForVersion(self.version)
        # Create handler instance
        self.handler = handlerclass()
        # Create parser instance that uses the given handler
        self.parser = pykickstart.parser.KickstartParser(self.handler)

    def read(self,ksfile,reset=True):
        '''Read in the given kickstart file. If reset is True, the handler
        and parser will be re-initialized based on the contents of the file.'''
        f = open(ksfile)
        self.readfp(f,reset)
        f.close()

    def readfp(self,fp,reset=True):
        '''Read kickstart data from the given file-like object.
        (fp must support readlines())
        Set the 'reset' argument to False if you don't want to re-initialize
        the parser, handler, etc.'''
        # FIXME: need to ignore deprecation warnings etc.
        data = list()
        for l in fp.readlines():
            data.append(l)
            if reset:
                if l.startswith("# Description: "):
                    (k,v) = l.split(":",1)
                    self.description = v.strip()
                elif l.startswith("# Version: "):
                    (k,v) = l.split(":",1)
                    self.version = int(v.strip())
        if reset:
            self.setversion(self.version)
        self.parser.readKickstartFromString(''.join(data))

    def handlecommand(self,cmdstring):
        '''Given a kickstart command (in a string), pass that to the kickstart
        parser and modify the kickstart object accordingly.'''
        self.parser.handleCommand(0,shlex.split(cmdstring))

    def write(self,ksfile,handler=None):
        '''Write a valid kickstart file out to the given filename.'''
        f = open(ksfile,'w')
        self.writefp(f,handler)
        f.close()

    def writefp(self,fp,handler=None):
        '''Write valid kickstart data out to the given file-like object.'''
        fp.write(self.__str__(handler))

    # This needs to know what vintage of kickstart to write out. If handler is
    # left as None, we use our default handler.
    def __str__(self,handler=None):
        h = header % (self.description, self.version)
        if handler:
            print "FIXME: can't actually convert between handlers yet"
            handler = self.handler
        else:
            handler = self.handler
        return h + str(handler)
