#!/usr/bin/python
#
# Copyright 2007 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import logging

from sqlobject import *
sqlhub.processConnection = connectionForURI("sqlite:/:memory:")

# Create object structure
from snake.xmlhelper import indent,ET
from snake.compose import Compose
from snake.tree import ComposeTarget,Tree,Source,IsoSet,Debug
from sqlobject import *
from sqlobject.inheritance import InheritableSQLObject

def createTables():
    SOCompose.createTable()
    SOComposeTarget.createTable()
    SODebug.createTable()
    SOIsoSet.createTable()
    SOSource.createTable()
    SOTree.createTable()
    SOImage.createTable()
    SOImageFile.createTable()

    # Create regexp function for sqlite 
    import re
    def func_regexp(expr, item):
        r = re.compile(expr, re.IGNORECASE)
        return (item and r.match(item)) is not None

    conn = sqlhub.processConnection.getConnection()
    conn.create_function("regexp", 2, func_regexp)

def clearTrees():
    '''Delete all Tree, Source, IsoSet, Debug, and Compose records
       This is typically called when refreshing the tree database
    '''

    from sqlobject.sqlbuilder import Delete
    conn = sqlhub.processConnection
    conn.query(conn.sqlrepr(Delete(SOImageFile.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOImage.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOTree.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SODebug.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOIsoSet.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOSource.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOComposeTarget.sqlmeta.table, None)))
    conn.query(conn.sqlrepr(Delete(SOCompose.sqlmeta.table, None)))

def dropTables():
    SOCompose.dropTable()
    SOComposeTarget.dropTable()
    SODebug.dropTable()
    SOIsoSet.dropTable()
    SOSource.dropTable()
    SOImageFile.dropTable()
    SOImage.dropTable()
    SOTree.dropTable()

def importTreesFromDir(dirlist, xmlcachedir=None):
    '''Finds trees found under the give list of directories
       Returns the number of trees imported
    '''

    import logging
    log = logging.getLogger("snake")

    treeCount=0
    for dir in dirlist:
        log.debug(dir)
        if xmlcachedir:
            # N.B. os.path.join doesn't like joining two absolute paths
            xmldir=xmlcachedir + dir
            if os.path.exists(xmldir+'/.compose.xml'):
                c=SOCompose()
                c.loaddir(xmldir)
                for t in c.trees:
                    log.debug("  +-- " + t.reldir)
                    t.absdir = os.path.join(dir, t.reldir)
                    treeCount = treeCount + 1
            elif os.path.exists(xmldir+'/.tree.xml'):
                t=SOTree()
                t.loaddir(xmldir)
                t.absdir = dir
                treeCount = treeCount + 1
        if os.path.exists(os.path.join(dir,'.composeinfo')):
            c=SOCompose()
            c.loaddir(dir)
            for t in c.trees:
                log.debug("  +-- " + t.reldir)
                t.absdir = os.path.join(dir, t.reldir)
                treeCount = treeCount + 1
        elif os.path.exists(os.path.join(dir,'.treeinfo')):
            t = SOTree()
            t.loaddir(dir)
            t.absdir = dir
            treeCount = treeCount + 1

    return treeCount 

def getContainers(dirList):
    '''glob expland the snake.conf containers list'''

    import glob

    containers = []
    for c in dirList:
        containers.extend(glob.glob(c))
    return containers


class SOCompose(SQLObject,Compose):

    name = StringCol(default=None)
    time = FloatCol(default=None)

    trees = MultipleJoin('SOTree', joinColumn="compose_id")
    isos = MultipleJoin('SOIsoSet', joinColumn="compose_id")
    debug = MultipleJoin('SODebug', joinColumn="compose_id")
    source = MultipleJoin('SOSource', joinColumn="compose_id")

    def __init__(self, **kwargs):
        SQLObject.__init__(self)
        #Compose.__init__(self, **kwargs)

    def __repr__(self):
        return "Compose(%s)" % self.id

    def __str__(self):
        return "%s%s" % (self.id, str(self.trees))

    def parsecomposeinfo(self,composeinfo):
        f=open(composeinfo)
        self.name=os.path.basename(os.path.dirname(composeinfo))
        for line in f.readlines():
            (type,info) = line.split(":")
            if type == 'tree':
                dir = os.path.join(os.path.dirname(composeinfo),info.strip())
                t=SOTree()
                t.reldir = info.strip()
                t.composeID=self.id
                t.loaddir(dir)
            # FIXME: add other types

    def parsexml(self,xml):
        xmldir=os.path.dirname(xml)
        elemtree = ET.parse(xml)
        r=elemtree.getroot()
        self.name=r.get('name')
        self.time=float(r.get('time'))
        typemap={'debug' :(SODebug,self.debug),
                 'source':(SOSource,self.source),
                 'tree'  :(SOTree,self.trees),
                 'cdset' :(SOIsoSet,self.isos),
                 'dvdset':(SOIsoSet,self.isos)}
        for (type, (obj, set)) in typemap.items():
            for e in r.findall(type):
                c=obj()
                c.arch=e.get('arch')
                c.variant=e.get('variant') or ""
                c.reldir=e.text
                try:
                    c.parsexml(os.path.join(xmldir,c.reldir,".%s.xml" % type))
                except:
                    pass
                c.composeID=self.id
        return elemtree

class SOComposeTarget(InheritableSQLObject):
    '''A generic base class for trees and other things built in a compose'''

    name = StringCol(default=None)
    time = FloatCol(default=None)
    family = StringCol(default=None)
    variant = StringCol(default=None)
    version = StringCol(default=None)
    arch = StringCol(default=None)
    reldir = StringCol(default=None)
    absdir = StringCol(default=None)

    def __init__(self, **kwargs):
        SQLObject.__init__(self, **kwargs)

class SOImageFile(SQLObject):
    ftype = StringCol()
    path = StringCol()
    image = ForeignKey('SOImage', notNone=True)

class SOImage(SQLObject):
    ftype = StringCol()
    path = StringCol()
    arch = StringCol(default=None)
    tree = ForeignKey('SOTree', dbName="tree_id", notNone=True)
    #files = MultipleJoin('ImageFile')

    def __repr__(self):
        return "%s" % {"type": self.ftype, "path": self.path, "arch": self.arch}

class SOTree(SOComposeTarget, ComposeTarget):
    '''A class that represents an installable tree'''
    compose = ForeignKey('SOCompose', dbName="compose_id", default=None)
    images = MultipleJoin('SOImage', joinColumn="tree_id")

    # This subclass adds the 'images' dict, which contains boot images
    def __init__(self,**kwargs):
        SOComposeTarget.__init__(self,**kwargs)

    def getImageDict(self):
        '''Used when marshalling the SOImage objects is required (e.g. XMLRPC)'''
        imgDict = dict()
        for img in self.images:
            if not imgDict.has_key(img.arch):
                imgDict[img.arch] = dict()
            imgDict[img.arch][img.ftype] = img.path
        return imgDict

    def parsetreeinfo(self,treeinfo):
        c=ComposeTarget.parsetreeinfo(self,treeinfo)
        for s in c.sections():
            if s.startswith('images-'):
                for ftype, path in c.items(s):
                    SOImage(arch=s.replace('images-',''), ftype=ftype, path=path, tree=self.id)

    def parsexml(self,xml):
        elemtree=ComposeTarget.parsexml(self,xml)
        r=elemtree.getroot()
        # FIXME: This doesn't work!!
        for image_elem in r.findall('images'):
            for e in image_elem.findall('file'):
                SOImage(arch=image_elem.get('arch'), ftype=e.get('type'), path=e.text, tree=self.id)

class SOIsoSet(SOComposeTarget, ComposeTarget):
    '''A class that represents a set of ISO images'''
    # This class has a list of iso images, and the boot images live therein.
    # FIXME: We'll need an ISO helper to pull info out of the disc images.
    compose = ForeignKey('SOCompose', dbName="compose_id", default=None)

# Source and Debug are two other things we might need to consider here.
# These are just stubs, defined so we don't forget about them.

class SOSource(SOComposeTarget, ComposeTarget):
    '''A class for a SRPM dir'''
    compose = ForeignKey('SOCompose', dbName="compose_id", default=None)

class SODebug(SOComposeTarget, ComposeTarget):
    '''A class for a debuginfo dir'''
    compose = ForeignKey('SOCompose', dbName="compose_id", default=None)


if __name__ == "snake.server":
    createTables()
