#!/usr/bin/python
# snake/treedb.py - functions for manipulating the tree database

import sys, os, time, tempfile
import rpm
import snake.uri
import snake.tree
from ConfigParser import SafeConfigParser

treedb_dir = '/var/lib/snake/trees' # FIXME config file

def _loadtrees(dir=treedb_dir,check=True):
    '''Loads all the treeinfo files in the given dir.
    Returns a list of Tree objects.
    If check is True, any treeinfo file lacking URIs will be removed from the
    treedb.'''
    treelist = list()
    for name in os.listdir(dir):
        f = os.path.join(dir,name)
        t = snake.tree.Tree()
        t.parsetreeinfo(f)
        if check and not (hasattr(t,'uris') and t.uris):
            removetree(t.id)
        treelist.append(t)
    return treelist

def _writetree(tree):
    '''Write the given Tree object back to the tree database'''
    # FIXME: safen tree.id
    treefile = os.path.join(treedb_dir,"%s.treeinfo" % tree.id)
    fp = open(treefile,"w")
    tree.config.write(fp)
    fp.close()

def id_to_tree(id):
    treefile = os.path.join(treedb_dir,"%s.treeinfo" % id)
    if not os.path.exists(treefile):
        return None
    try:
        t = snake.tree.Tree()
        t.parsetreeinfo(treefile)
    except ConfigParser.NoSectionError:
        # File exists but there's nobody home. Bluh.
        return None
    return t

def lookup_tree_by_uri(uri):
    '''Given a uri, search the local treedb for the Tree object containing it.
    Return a Tree object, or None on failure.'''
    treelist = _loadtrees()
    tree = None
    for t in treelist:
        if uri in t.uris:
            tree = t
            break
    return t

def add(uri):
    '''Add the given URI to the tree database'''
    # Get the new tree's info

    try: 
        return _add_treeinfo_uri(uri)
    except:
        return _add_discinfo_uri(uri)

def _add_treeinfo_uri(uri):
    '''Add the given treeinfo URI to the tree database'''

    ti = snake.uri.treeinfo(uri)
    c = SafeConfigParser()
    c.readfp(ti)
    # Missing UUID. Boo! Fake it with the timestamp.
    if not c.has_option('general','id'):
        c.set('general','id',c.get('general','timestamp'))
    # Check to see if we've seen this tree or URI before.
    uris = list()
    # FIXME safen tree.id - malicious tree could have bad stuff in there
    treefile = os.path.join(treedb_dir,"%s.treeinfo" % c.get('general','id'))
    if os.path.exists(treefile):
        # It does. Read that info too.
        c.read(treefile)
        if c.has_option('general','uris'):
            uris = c.get('general','uris').split()
    # Add our URI to the list, if it's not there already
    if uri not in uris:
        uris.append(uri)
        c.set('general','uris',"\n".join(uris))
        # And write it out.
        c.write(open(treefile,"w"))
    return True

def _add_discinfo_uri(uri):
    '''Add the given discinfo URI to the tree database.  This generally is 
    executed for older style trees that do not contain a .treeinfo file'''

    # load .discinfo uri
    di = snake.uri.discinfo(uri)
    
    # populate a tree object with di
    t = snake.tree.Tree()
    t.parsediscinfo(di)
    di.close()

    # Over ride the timestamp and variant based on presence of a comps.rpm
    if False and not t.variant:
        '''HACK - use comps.rpm to determine the variant and proper timestamp.
        This works but fails when running snake-tree update since that code path 
        instantiates the tree using the .discinfo URI.  This URI does not 
        include the comps.rpm HACK and results in a different tree.id'''
        try:
            fd = snake.uri.uriopen(os.path.join(uri,"RedHat/base/comps.rpm"))
        except snake.uri.URIError, e:
            pass
        else:
            (tmpfd,tmpcomps) = tempfile.mkstemp('.comps.rpm')

            # FIXME - __file_write should be in a more generic module
            from snake.install import __file_write
            b = __file_write(fd,open(tmpcomps,'wb'))

            # FIXME - should we use rpmUtils (yum) or rpm (rpm-python)?
            # import rpmUtils.transaction, rpmUtils.miscutils
            # ts = rpmUtils.transaction.initReadOnlyTransaction()
            # h = rpmUtils.miscutils.hdrFromPackage(ts, tmpfd)
            ts = rpm.TransactionSet()
            ts.setVSFlags((rpm._RPMVSF_NOSIGNATURES|rpm.RPMVSF_NOMD5|rpm.RPMVSF_NEEDPAYLOAD))
            h = ts.hdrFromFdno(tmpfd)

            # comps %{version} is often 4AS or 4Desktop ... this will remove the version
            t.id = str(h[rpm.RPMTAG_BUILDTIME])
            t.time = str(h[rpm.RPMTAG_BUILDTIME])
            t.variant = h[rpm.RPMTAG_VERSION].strip(t.version)

            fd.close()
            os.close(tmpfd)
            os.unlink(tmpcomps)

    # build ConfigParser object with tree information
    c = SafeConfigParser()

    treefile = os.path.join(treedb_dir,"%s.treeinfo" % t.id)
    if os.path.exists(treefile):
        c.read(treefile)
    else:
        c.add_section("general")

    # record general options
    for attr in ("id", "family", "variant", "version", "arch"):
        if hasattr(t,attr):
            c.set("general", attr, getattr(t,attr))

    c.set("general", "timestamp", str(t.time))

    if c.has_option('general','uris'):
        uris = c.get('general','uris').split()
    else:
        uris = []

    # record boot image paths
    if hasattr(t,"images"):
        for (arch,boot_images) in t.images.items():
            section = "images-%s" % arch
            if not c.has_section(section):
                c.add_section(section)

            for (type,paths) in boot_images.items():
                if isinstance(paths,str):
                    '''were we given a single string or a list of paths?'''
                    paths = [paths]

                for path in paths:
                    try:
                        '''check if the image exists'''
                        fd = snake.uri.uriopen(os.path.join(uri,path))
                    except snake.uri.URIError:
                        '''not a valid image uri'''
                        continue
                    else:
                        '''valid uri found ... record and move on'''
                        fd.close()
                        c.set(section,type,path)
                        break

            # if no valid images were found, remove the section
            if not c.options(section):
                c.remove_section(section)
    
    if not hasattr(t,"images") or \
        True not in map(lambda s: s.startswith("images-"), c.sections()): 
        '''this is not a bootable tree'''
        return False

    # update uris
    if uri not in uris:
        uris.append(uri)
        c.set('general','uris',"\n".join(uris))
        c.write(open(treefile,"w"))

    return True

def listids(dir=treedb_dir):
    '''Return a list of known tree IDs.'''
    idlist = list()
    for name in os.listdir(dir):
        (id,ext) = os.path.splitext(name)
        if ext != '.treeinfo':
            continue
        idlist.append(id)
    return idlist

def listtrees(**args):
    '''Return a list of Trees that match all the given args. Recognized args:
    id:     Tree id
    uri:    URI to a tree
    arch:   architecture
    family: family name (e.g. Fedora)
    ...and basically any other attribute of the Tree object.
    '''
    treelist = _loadtrees()
    resultlist = list()
    for t in treelist:
        ok = True
        for (k,v) in args.items():
            if getattr(t,k) != v:
                ok = False
                break
        if ok:
            resultlist.append(t)
    return resultlist

def removeuri(uri):
    '''Remove the given URI from the tree database'''
    t = lookup_tree_by_uri(uri)
    if uri in t.uris:
        t.uris.remove(uri)
    # Tree has no remaining URIs - remove it.
    if not t.uris:
        removetree(t.id)
    else:
        # FIXME this needs to happen in the tree object somehow
        t.config.set('general','uris',"\n".join(t.uris))
        _writetree(t)
    
def removetree(id):
    '''Remove the given tree from the tree database'''
    # This is actually pretty easy.
    os.unlink(os.path.join(treedb_dir,"%s.treeinfo" % id))
